# SCOPE

To install the code run `pip install -e .` in the root directory.

The code uses `hydra` for configuration management. The configuration files are located in `scope/configs`. There are different config files for training and evaluation. All the hyperparameters are defined in the config file `scope/scope/alignment/configs.py` and can be modified through the `.yaml` files in `scope/configs`.


# Training

Several training modes are supported through the following scripts:
- `scope/scope/run_spin.py` for standard fine-tuning models.
- `scope/scope/run_scope.py` for traning using the SCOPE preference tuning method.
- `scope/scope/run_cliff.py` for training using the CLIFF contrastive loss.

# Generation

Generation is performed using the script `scope/scope/generate.py`. Several generation modes are supported:
- passing `model_type=standalone` will generate samples using standard decoding.
- passing `model_type=mixture` will generate samples using a main model and an auxiliary model. This is the option that should be chosen either for generating our unfaithful samples or for generating samples using CAD, PMI or CRITIC method.
- The mixture should be further specified with the argument `generation.mixture_mode` which can be `cad, pmi, critic, unfaithful`.

For instance, generating unfaithful samples with $\alpha=0.5$ can be done with the following command:
```bash
python scope/scope/generate.py --config-name d2t model=llama7b model.model_path=/path/to/finetuned/model model_type=mixture model@noise=llama7b generation.mixture_mode=unfaithful eval.dtype=bf16 eval.batch_size=4 task=totto-gen data=totto do_gen=true  generation.mixture_alpha=0.5 generation=llama generation.do_sample=true
```

# CRITIC
Code to train the CRITIC model is located in `scope/scope/critic` and were adapted from the original implementation of CRITIC https://github.com/langus0/critic-aware-decoding.


# Data download
The code to download the data is located in `scope/data/reformat.py`.